
def dump_log(ram_write, ram_read, bank, pc):
    for b in sorted(bank.keys()):
        print("Bank %02X at %04X" % (b,bank[b]))
    for p in sorted(pc):
        print("PC: "+p)
    # print contiguous ram write regions
    mem = [0] * 0x8000
    for a in ram_write:
        mem[a] = 1
    pos = 0
    ms = -1
    while pos < 0x8000:
        if mem[pos] == 1:
            if (ms < 0):
                ms = pos
        else:
            if (ms >= 0):
                print("RAM range W: %04X - %04X" % (ms,pos-1))
            ms = -1
        pos += 1
    # print contiguous ram read regions
    mem = [0] * 0x8000
    for a in ram_read:
        mem[a] = 1
    pos = 0
    ms = -1
    while pos < 0x8000:
        if mem[pos] == 1:
            if (ms < 0):
                ms = pos
        else:
            if (ms >= 0):
                print("RAM range R: %04X - %04X" % (ms,pos-1))
            ms = -1
        pos += 1

def analyze(log):
    print("Analyzing: "+log)
    f = open(log,"rt")
    ram_write = set()
    ram_read = set()
    bank = dict()
    pc = set()
    warns = set()
    played = False
    for line in f:
        if line[0:5] == "BEGIN":
            dump_log(ram_write,ram_read,bank,pc)
            ram_write = set()
            ram_read = set()
            bank = dict()
            pc = set()
            warns = set()
            played = False
            print(line)
        elif line[0:4] == "BANK":
            a = int(line[5:9],16)
            b = int(line[10:12],16)
            if played and (b in bank):
                c = a | (b << 16)
                if a != bank[b] and c not in warns:
                    print("Warning: bank %02X also at %04X" % (b,bank[b]))
                    warns.add(c)
            bank[b] = a
        elif line[0:5] == "WRITE":
            a = int(line[6:10],16)
            ram_write.add(a)
        elif line[0:4] == "READ":
            a = int(line[5:9],16)
            ram_read.add(a)
        elif line[0:9] == "SWITCH_PC":
            a = line[15:17] + ':' + line[10:14]
            pc.add(a)
        elif line[0:4] == "PLAY":
            played = True

#analyze("nsf_write_00_06.log")    
#analyze("nsf_write_07_09.log")    
#analyze("nsf_write_10_13.log")    
#analyze("nsf_write_14_16.log")
#analyze("nsf_write_redo.log")
analyze("nsf_write_chipchop_fix.log")
